package org.springblade.resouces;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.exceptions.ExceptionUtil;
import cn.hutool.core.exceptions.ValidateException;
import cn.hutool.core.util.ObjectUtil;
import com.alibaba.fastjson2.JSONArray;
import com.alibaba.fastjson2.JSONObject;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import lombok.extern.slf4j.Slf4j;
import org.springblade.bi.client.form.AppCreateDatasourceParams;
import org.springblade.core.secure.utils.AuthUtil;
import org.springblade.core.secure.utils.SecureUtil;
import org.springblade.cube.model.*;
import org.springblade.cube.service.*;
import org.springblade.datasource.DsHandler;
import org.springblade.datasource.enums.DsTypeEnum;
import org.springblade.datasource.model.TableViewDto;
import org.springblade.datasource.support.DynamicDataSourceSupport;
import org.springblade.report.entity.*;
import org.springblade.report.service.IReportDatasourceService;
import org.springblade.report.service.impl.DimensionServiceImpl;
import org.springblade.report.service.impl.TableCategoryServiceImpl;
import org.springblade.report.service.normal.ReportDatasourcePermissionService;
import org.springblade.system.client.AppClient;
import org.springblade.system.client.model.PlatomixAppCopyLogModel;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * @author 喜欢番茄火锅
 * @date 2024/9/3 17:07
 */
@Service
@Slf4j
public class AppCopyService {

    @Autowired
    private CubeDataService cubeDataService;

    @Autowired
    private CubeDataPermissionService cubeDataPermissionService;

    @Autowired
    private CubeTableColumnService cubeTableColumnService;

    @Autowired
    private CubeTableStaticService cubeTableStaticService;

    @Autowired
    private IReportDatasourceService reportDatasourceService;

    @Autowired
    private CubeSqlViewService cubeSqlViewService;

    @Autowired
    private CubeTableFileService cubeTableFileService;

    @Autowired
    private DimensionServiceImpl dimensionService;

    @Autowired
    private TableCategoryServiceImpl tableCategoryService;

    @Autowired
    private ReportDatasourcePermissionService reportDatasourcePermissionService;

    @Autowired
    private DsHandler dsHandler;

    @Resource
    private AppClient appClient;

    @Autowired
    private DynamicDataSourceSupport dynamicDataSourceSupport;

    /**
     * 复制表单涉及BI相关内容
     * @param cubeIds
     * @param dsIds
     * @param appId
     * @param targetAppId
     * @param targetAppDsId
     * @param platomixAppCopyLogModel
     */
    @Async
    @Transactional
    public Boolean copy(List<Long> cubeIds, List<Long> dsIds, String appId, String targetAppId, Long targetAppDsId, PlatomixAppCopyLogModel platomixAppCopyLogModel, List<String> associationTables) {
        log.info("*********************应用表单复制 START***********************");
        log.info("cubeIds:{}, dsIds:{}, appId:{}, targetAppId:{}", cubeIds.toString(), dsIds.toString(), appId, targetAppId);
        log.info("associationTables:{}", associationTables.toString());
        boolean result = false;
        log.info("1 复制数据源相关(包括物理表)->start");
        if(dsIds.size()>=1){
            result = copyDs(dsIds, cubeIds, appId, targetAppId, targetAppDsId, platomixAppCopyLogModel);
        }
        log.info("1 复制数据源相关-->end");
        if(result){
            platomixAppCopyLogModel.setDsResult(0);
        }else{
            platomixAppCopyLogModel.setDsResult(1);
            platomixAppCopyLogModel.setErrorInfo("复制数据源表结构失败.");
        }
        appClient.taskInfo(platomixAppCopyLogModel);
        try {
            if(result){
                log.info("2 复制数据集相关-->start");
                copyCube(cubeIds, appId, targetAppId, targetAppDsId);
                platomixAppCopyLogModel.setCubeResult(0);
                appClient.taskInfo(platomixAppCopyLogModel);
                log.info("2 复制数据集相关-->end");

                //复制关联物理表结构
                log.info("3 复制关联物理表-->start");
                copyAssociationTables(dsIds, targetAppDsId, associationTables, platomixAppCopyLogModel);
                log.info("3 复制关联物理表-->end");
            }
            return result;
        } catch (Exception e) {
            platomixAppCopyLogModel.setCubeResult(1);
            platomixAppCopyLogModel.setErrorInfo(ExceptionUtil.stacktraceToString(e));
            appClient.taskInfo(platomixAppCopyLogModel);
            log.error("数据源 复制失败:" + ExceptionUtil.stacktraceToString(e));
            return result;
        }
    }


    /**
     * 删除表单涉及BI相关内容
     * @param cubeIds
     * @param dsIds
     * @param appId
     * @param platomixAppCopyLogModel
     * @return Boolean
     */
    public Boolean del(List<Long> cubeIds, List<Long> dsIds, String appId, PlatomixAppCopyLogModel platomixAppCopyLogModel) {
        log.info("*********************应用表单删除 START***********************");
        log.info("cubeIds:{}, dsIds:{}, appId:{}", cubeIds.toString(), dsIds.toString(), appId);
        log.info("platomixAppCopyLogModel:{}", platomixAppCopyLogModel.toString());
        boolean result = false;
        log.info("1 删除数据源相关-->start");
        if(dsIds.size()>=1){
            result = delDs(dsIds, appId, platomixAppCopyLogModel);
        }
        log.info("1 删除数据源相关-->end");
        if(result){
            platomixAppCopyLogModel.setDsResult(0);
        }else{
            platomixAppCopyLogModel.setDsResult(1);
            platomixAppCopyLogModel.setErrorInfo("复制数据源表结构失败.");
        }
        appClient.taskInfo(platomixAppCopyLogModel);
        try {
            if(result){
                log.info("2 删除数据集结构相关-->start");
                String targetAppId = null;
                Long targetAppDsId = null;
                delCube(cubeIds, appId, targetAppId, targetAppDsId);
                log.info("2 删除数据集结构相关-->end");
                platomixAppCopyLogModel.setCubeResult(0);
                appClient.taskInfo(platomixAppCopyLogModel);
            }
            return result;
        } catch (Exception e) {
            platomixAppCopyLogModel.setCubeResult(1);
            platomixAppCopyLogModel.setErrorInfo(ExceptionUtil.stacktraceToString(e));
            appClient.taskInfo(platomixAppCopyLogModel);
            log.error("数据源 删除失败:" + ExceptionUtil.stacktraceToString(e));
            return result;
        }
    }


    /**
     * 复制数据源表结构
     * @param dsIds     数据源ID
     * @param cubeIds   数据集ID
     * @param appId
     * @param targetAppId
     * @param targetAppDsid
     * @param platomixAppCopyLogModel
     * @reurn Boolean
     */
    private Boolean copyDs(List<Long> dsIds, List<Long> cubeIds,  String appId, String targetAppId, Long targetAppDsid, PlatomixAppCopyLogModel platomixAppCopyLogModel) {
        //复制数据集物理表结构
        if (!reportDatasourceTableService(dsIds, cubeIds, appId, targetAppDsid, platomixAppCopyLogModel)) {
            log.error("复制数据集物理表结构返回为false.");
            //return false;
        }
        cubeSqlViewService(dsIds, appId, targetAppId, targetAppDsid, 0);
        log.info("1.1 数据集_SQL视图结构 复制完成");
        cubeTableFileService(dsIds, appId, targetAppId, targetAppDsid, 0);
        log.info("1.2 文件表 复制完成");
        dimensionService(dsIds, appId, targetAppId, targetAppDsid, 0);
        log.info("1.3 报表维度实体类 复制完成");
        reportDatasourcePermissionService(dsIds, appId, targetAppId, targetAppDsid, 0);
        log.info("1.4 数据源权限组 复制完成");
        tableCategoryService(dsIds, appId, targetAppId, targetAppDsid, 0);
        log.info("1.5 表分类实体类 复制完成");
        return true;
    }


    /**
     * 删除数据源表结构
     * @param dsIds
     * @param appId
     * @param platomixAppCopyLogModel
     * @reurn Boolean
     */
    private Boolean delDs(List<Long> dsIds, String appId, PlatomixAppCopyLogModel platomixAppCopyLogModel) {
        String targetAppId = null;
        Long targetAppDsid = null;
        cubeSqlViewService(dsIds, appId, targetAppId, targetAppDsid, 1);
        log.info("1.1 数据集_SQL视图结构 删除完成");
        cubeTableFileService(dsIds, appId, targetAppId, targetAppDsid, 1);
        log.info("1.2 文件表 删除完成");
        dimensionService(dsIds, appId, targetAppId, targetAppDsid, 1);
        log.info("1.3 报表维度实体类 删除完成");
        reportDatasourcePermissionService(dsIds, appId, targetAppId, targetAppDsid, 1);
        log.info("1.4 数据源权限组 删除完成");
        tableCategoryService(dsIds, appId, targetAppId, targetAppDsid, 1);
        log.info("1.5 表分类实体类 删除完成");
        return true;
    }


    /**
     * 复制【表分类实体类】
     * @param dsIds
     * @param appId
     * @param targetAppId
     * @param targetAppDsid
     * @param type
     * @return void
     */
    private void tableCategoryService(List<Long> dsIds, String appId, String targetAppId, Long targetAppDsid, Integer type) {
        List<TableCategory> list = tableCategoryService.lambdaQuery()
                .eq(TableCategory::getAppId, appId)
                .in(TableCategory::getDatasourceId, dsIds).list();
        if(CollectionUtil.isEmpty(list)){
            log.info("tableCategoryService 无相关信息,不需要操作.");
            return;
        }
        switch (type) {
            case 0:
                for (TableCategory obj : list) {
                    TableCategory tableCategory = tableCategoryService.lambdaQuery()
                            .eq(TableCategory::getAppId, appId)
                            .eq(TableCategory::getDatasourceId, obj.getDatasourceId())
                            .eq(TableCategory::getId, obj.getId()).one();
                    if(ObjectUtil.isEmpty(tableCategory)){
                        obj.setKid(null);
                        obj.setAppId(targetAppId);
                        obj.setDatasourceId(targetAppDsid);
                        obj.setCreateTime(new Date());
                        tableCategoryService.save(obj);
                    }else{
                        log.error("tableCategoryService 相关信息已经存在:{},不需要进行复制.", obj.toString());
                    }
                }
                break;
            case 1:
                for (TableCategory obj : list) {
                    tableCategoryService.removeById(obj.getKid());
                }
                break;
            default:
                throw new ValidateException("参数错误");
        }
    }

    /**
     * 复制【数据源权限组】
     * @param dsIds
     * @param appId
     * @param targetAppId
     * @param targetAppDsid
     * @param type
     * @return void
     */
    private void reportDatasourcePermissionService(List<Long> dsIds, String appId, String targetAppId, Long targetAppDsid, Integer type) {
        List<ReportDatasourcePermission> list = reportDatasourcePermissionService.lambdaQuery()
                .eq(ReportDatasourcePermission::getAppId, appId)
                .in(ReportDatasourcePermission::getDatasourceId, dsIds).list();
        if(CollectionUtil.isEmpty(list)){
            log.info("reportDatasourcePermissionService 无相关信息,不需要操作.");
            return;
        }
        switch (type) {
            case 0:
                for (ReportDatasourcePermission obj : list) {
                    ReportDatasourcePermission permission = reportDatasourcePermissionService.lambdaQuery()
                            .eq(ReportDatasourcePermission::getAppId, appId)
                            .eq(ReportDatasourcePermission::getDatasourceId, obj.getDatasourceId())
                            .eq(ReportDatasourcePermission::getId, obj.getId()).one();
                    if(ObjectUtil.isEmpty(permission)){
                        obj.setKid(null);
                        obj.setAppId(targetAppId);
                        obj.setDatasourceId(targetAppDsid);
                        obj.setCreateTime(new Date());
                        obj.setCreateUserId(AuthUtil.getUserId());
                        reportDatasourcePermissionService.save(obj);
                    }else{
                        log.error("reportDatasourcePermissionService 相关信息已经存在:{},不需要进行复制.", obj.toString());
                    }
                }
                break;
            case 1:
                for (ReportDatasourcePermission obj : list) {
                    reportDatasourcePermissionService.removeById(obj.getKid());
                }
                break;
            default:
                throw new ValidateException("参数错误");
        }
    }

    /**
     * 复制【报表维度实体类】
     * @param dsIds
     * @param appId
     * @param targetAppId
     * @param targetAppDsid
     * @param type
     * @return void
     */
    private void dimensionService(List<Long> dsIds, String appId, String targetAppId, Long targetAppDsid, Integer type) {
        List<Dimension> list = dimensionService.lambdaQuery()
                .eq(Dimension::getAppId, appId)
                .in(Dimension::getDatasourceId, dsIds).list();
        if(CollectionUtil.isEmpty(list)){
            log.info("dimensionService 无相关信息,不需要操作.");
            return;
        }
        switch (type) {
            case 0:
                for (Dimension obj : list) {
                    Dimension dimension = dimensionService.lambdaQuery()
                            .eq(Dimension::getAppId, appId)
                            .eq(Dimension::getDatasourceId, obj.getDatasourceId())
                            .eq(Dimension::getId, obj.getId()).one();
                    if(ObjectUtil.isEmpty(dimension)){
                        obj.setKid(null);
                        obj.setAppId(targetAppId);
                        obj.setDatasourceId(targetAppDsid);
                        obj.setCreateTime(new Date());
                        dimensionService.save(obj);
                    }else{
                        log.error("dimensionService 相关信息已经存在:{},不需要进行复制.", obj.toString());
                    }
                }
                break;
            case 1:
                for (Dimension obj : list) {
                    dimensionService.removeById(obj.getKid());
                }
                break;
            default:
                throw new ValidateException("参数错误");
        }
    }

    /**
     * 复制【文件表】
     * @param  dsIds  数据源ID
     * @param  appId  appId
     * @param  targetAppId 目标应用ID
     * @param  targetAppDsId
     * @param  type
     * @return void
     */
    private void cubeTableFileService(List<Long> dsIds, String appId, String targetAppId, Long targetAppDsId, Integer type) {
        List<CubeTableFile> list = cubeTableFileService.lambdaQuery()
                .eq(CubeTableFile::getAppId, appId)
                .in(CubeTableFile::getDatasourceId, dsIds).list();
        if(CollectionUtil.isEmpty(list)){
            log.info("cubeTableFileService 无相关信息,不需要操作.");
            return;
        }
        switch (type) {
            case 0:
                for (CubeTableFile obj : list) {
                    CubeTableFile cubeTableFile = cubeTableFileService.lambdaQuery()
                            .eq(CubeTableFile::getAppId, appId)
                            .eq(CubeTableFile::getDatasourceId, obj.getDatasourceId())
                            .eq(CubeTableFile::getId, obj.getId()).one();
                    if(ObjectUtil.isEmpty(cubeTableFile)){
                        obj.setKid(null);
                        obj.setAppId(targetAppId);
                        obj.setDatasourceId(targetAppDsId);
                        obj.setCreateTime(new Date());
                        obj.setCreateUserId(AuthUtil.getUserId());
                        cubeTableFileService.save(obj);
                    }else{
                        log.error("cubeTableFileService 相关信息已经存在:{},不需要进行复制.", obj.toString());
                    }
                }
                break;
            case 1:
                for (CubeTableFile obj : list) {
                    cubeTableFileService.removeById(obj.getKid());
                }
                break;
            default:
                throw new ValidateException("参数错误");
        }
    }

    /**
     * 复制【数据集_SQL视图结构 】
     * @param  dsIds  数据源ID
     * @param  appId  应用ID
     * @param  targetAppId 目标应用ID
     * @param  targetAppDsid
     * @param  type
     * @return void
     */
    private void cubeSqlViewService(List<Long> dsIds, String appId, String targetAppId, Long targetAppDsid, Integer type) {
        log.info("dsIds:{}, appId:{}, targetAppId:{}, targetAppDsid:{}, type:{}", dsIds.toString(), appId, targetAppId, targetAppDsid, type);
        List<CubeSqlView> list = cubeSqlViewService.lambdaQuery()
                .eq(CubeSqlView::getAppId, appId)
                .in(CubeSqlView::getDatasourceId, dsIds).list();
        if(CollectionUtil.isEmpty(list)){
            log.info("cubeSqlViewService 无相关信息,不需要操作.");
            return;
        }
        switch (type) {
            case 0:
                for (CubeSqlView obj : list) {
                    //如果该数据源已经存在该条视图，不进行新增
                    CubeSqlView cubeSqlView = cubeSqlViewService.lambdaQuery()
                            .eq(CubeSqlView::getAppId, appId)
                            .eq(CubeSqlView::getDatasourceId, obj.getDatasourceId())
                            .eq(CubeSqlView::getId, obj.getId()).one();
                    if(ObjectUtil.isEmpty(cubeSqlView)){
                        obj.setKid(null);
                        obj.setAppId(targetAppId);
                        obj.setDatasourceId(targetAppDsid);
                        obj.setCreateTime(new Date());
                        obj.setCreateUserId(AuthUtil.getUserId());
                        cubeSqlViewService.save(obj);
                    }else{
                        log.error("cubeSqlViewService相关信息已经存在:{},不需要进行复制.", obj.toString());
                    }
                }
                break;
            case 1:
                for (CubeSqlView obj : list) {
                    cubeSqlViewService.removeById(obj.getKid());
                }
                break;
            default:
                throw new ValidateException("参数错误");
        }
    }


    /**
     * 复制【数据源物理表】
     * @param  dsIds      数据源ID
     * @param  cubeIds    数据集ID
     * @param  appId      应用ID
     * @param  targetAppDsid 目标应用数据源ID
     * @param  platomixAppCopyLogModel  复制日志
     * @return boolean
     */
    private boolean reportDatasourceTableService(List<Long> dsIds,  List<Long> cubeIds, String appId, Long targetAppDsid, PlatomixAppCopyLogModel platomixAppCopyLogModel) {
        ReportDatasource reportDatasource = reportDatasourceService.getByDsId(targetAppDsid);
        List<TableViewDto> tableViewList = new ArrayList<>();
        List<CubeData> list = cubeDataService.lambdaQuery().in(CubeData::getId, cubeIds).in(CubeData::getDatasourceId, dsIds).eq(CubeData::getAppId, appId).list();
        if(CollectionUtil.isEmpty(list)){
            return false;
        }
        for(CubeData entity: list){
            String cubeName = entity.getCubeName();
            String cubeTableNames = entity.getCubeTableNames();
            JSONArray jsonArray = JSONArray.parseArray(cubeTableNames);
            TableViewDto tableViewDto = new TableViewDto();
            for(int i=0; i<jsonArray.size(); i++){
                JSONObject jsonObject = jsonArray.getJSONObject(i);
                String tableName = jsonObject.getString("tableName");
                tableViewDto.setTableViewName(tableName);
                tableViewDto.setTableComments(cubeName);
                tableViewDto.setDatasourceId(entity.getDatasourceId());
            }
            tableViewList.add(tableViewDto);
        }
        try {
            log.info("数据表 复制参数:{}", tableViewList.toString());
            dsHandler.tableCopy(reportDatasource, tableViewList);
            platomixAppCopyLogModel.setDbResult(0);
            appClient.taskInfo(platomixAppCopyLogModel);
            log.info("数据表 复制成功.");
            return true;
        } catch (Exception e) {
            platomixAppCopyLogModel.setDbResult(1);
            platomixAppCopyLogModel.setErrorInfo(ExceptionUtil.stacktraceToString(e));
            appClient.taskInfo(platomixAppCopyLogModel);
            log.error("数据表 复制失败:" + ExceptionUtil.stacktraceToString(e));
            return false;
        }
    }


    /**
     * 复制数据集相关内容
     * @param ids
     * @param appId
     * @param targetAppId
     * @param targetAppDsid
     * @return void
     */
    private void copyCube(List<Long> ids, String appId, String targetAppId, Long targetAppDsid) {
        cubeDataService(ids, appId, targetAppId, targetAppDsid, 0);
        log.info("2.1 数据集结构体复制完成");
        cubeDataPermissionService(ids, appId, targetAppId, 0);
        log.info("2.2 数据集权限复制完成");
        cubeTableColumnService(ids, appId, targetAppId, 0);
        log.info("2.3 数据集_表结构复制完成");
        cubeTableStaticService(ids, appId, targetAppId, targetAppDsid, 0);
        log.info("2.4 静态数据表记录复制完成");
    }


    /**
     * 删除数据集相关内容
     * @param ids
     * @param appId
     * @param targetAppId
     * @return void
     */
    private void delCube(List<Long> ids, String appId, String targetAppId, Long targetAppDsid) {
        cubeDataPermissionService(ids, appId, targetAppId, 1);
        log.info("2.2 数据集权限复制完成");
        cubeTableColumnService(ids, appId, targetAppId, 1);
        log.info("2.3 数据集_表结构复制完成");
        cubeTableStaticService(ids, appId, targetAppId, targetAppDsid, 1);
        log.info("2.4 静态数据表记录复制完成");
        cubeDataService(ids, appId, targetAppId, targetAppDsid, 1);
        log.info("2.1 数据集结构体复制完成");
    }




    /**
     * 复制【静态数据表记录】
     * @param ids
     * @param appId
     * @param targetAppId
     * @param targetAppDsid
     * @param type
     * @return void
     */
    private void cubeTableStaticService(List<Long> ids, String appId, String targetAppId, Long targetAppDsid, Integer type) {
        List<CubeTableStatic> list = cubeTableStaticService.lambdaQuery().eq(CubeTableStatic::getAppId, appId).in(CubeTableStatic::getCubeDataId, ids).list();
        if(CollectionUtil.isEmpty(list)){
            log.info("cubeTableStaticService 无相关信息,不需要操作.");
            return;
        }
        switch (type) {
            case 0:
                for (CubeTableStatic obj : list) {
                    CubeTableStatic findObject = cubeTableStaticService.lambdaQuery()
                            .eq(CubeTableStatic::getAppId, targetAppId)
                            .eq(CubeTableStatic::getDatasourceId, targetAppDsid)
                            .eq(CubeTableStatic::getId, obj.getId()).one();
                    if(ObjectUtil.isEmpty(findObject)){
                        obj.setKid(null);
                        obj.setAppId(targetAppId);
                        obj.setDatasourceId(targetAppDsid);
                        obj.setCreateTime(new Date());
                        obj.setCreateUserId(AuthUtil.getUserId());
                        cubeTableStaticService.save(obj);
                    }else{
                        log.error("cubeTableStaticService相关信息已经存在:{},不需要进行复制.", findObject.toString());
                    }
                }
                break;
            case 1:
                for (CubeTableStatic obj : list) {
                    cubeTableStaticService.removeById(obj.getKid());
                }
                break;
            default:
                throw new ValidateException("参数错误");
        }
    }

    /**
     * 复制【数据集_表结构】
     * @param ids
     * @param appId
     * @param targetAppId
     * @param type
     * @return void
     */
    private void cubeTableColumnService(List<Long> ids, String appId, String targetAppId, Integer type) {
        List<CubeTableColumn> list = cubeTableColumnService.lambdaQuery().eq(CubeTableColumn::getAppId, appId).in(CubeTableColumn::getCubeDataId, ids).list();
        if(CollectionUtil.isEmpty(list)){
            log.info("cubeTableColumnService 无相关信息,不需要操作.");
            return;
        }
        switch (type) {
            case 0:
                for (CubeTableColumn obj : list) {
                    CubeTableColumn findObject = cubeTableColumnService.lambdaQuery()
                            .eq(CubeTableColumn::getAppId, targetAppId)
                            .eq(CubeTableColumn::getId, obj.getId()).one();
                    if(ObjectUtil.isEmpty(findObject)){
                        obj.setKid(null);
                        obj.setAppId(targetAppId);
                        cubeTableColumnService.save(obj);
                    }else{
                        log.error("cubeTableColumnService相关信息已经存在:{},不需要进行复制.", findObject.toString());
                    }
                }
                break;
            case 1:
                for (CubeTableColumn obj : list) {
                    cubeTableColumnService.removeById(obj.getKid());
                }
                break;
            default:
                throw new ValidateException("参数错误");
        }
    }


    /**
     * 复制【数据集权限】
     * @param ids    数据集ID
     * @param appId  应用ID
     * @param targetAppId   目标应用ID
     * @param type   操作类型(1=删除/0=添加)
     * @return void
     */
    private void cubeDataPermissionService(List<Long> ids, String appId, String targetAppId, Integer type) {
        List<CubeDataPermission> list = cubeDataPermissionService.lambdaQuery().eq(CubeDataPermission::getAppId, appId).in(CubeDataPermission::getCubeDataId, ids).list();
        if(CollectionUtil.isEmpty(list)){
            log.info("cubeDataPermissionService 无相关信息,不需要操作.");
            return;
        }
        switch (type) {
            case 0:
                for (CubeDataPermission obj : list) {
                    CubeDataPermission findObject = cubeDataPermissionService.lambdaQuery()
                            .eq(CubeDataPermission::getAppId, targetAppId)
                            .eq(CubeDataPermission::getId, obj.getId()).one();
                    if(ObjectUtil.isEmpty(findObject)){
                        obj.setKid(null);
                        obj.setAppId(targetAppId);
                        obj.setCreateTime(new Date());
                        obj.setCreateUserId(AuthUtil.getUserId());
                        cubeDataPermissionService.save(obj);
                    }else{
                        log.error("cubeDataPermissionService相关信息已经存在:{},不需要进行复制.", findObject.toString());
                    }
                }
                break;
            case 1:
                for (CubeDataPermission obj : list) {
                    cubeDataPermissionService.removeById(obj.getKid());
                }
                break;
            default:
                throw new ValidateException("参数错误");
        }
    }


    /**
     * 复制【数据集结构体】
     * @param ids    数据集ID
     * @param appId  应用ID
     * @param targetAppId   目标应用ID
     * @param targetAppDsid 目标数据源ID
     * @param type  操作类型(1=删除/0=添加)
     * @return void
     */
    private void cubeDataService(List<Long> ids, String appId, String targetAppId, Long targetAppDsid, Integer type) {
        List<CubeData> list = cubeDataService.lambdaQuery().in(CubeData::getId, ids).eq(CubeData::getAppId, appId).list();
        if(CollectionUtil.isEmpty(list)){
            log.info("cubeDataService 无相关信息,不需要操作.");
            return;
        }
        switch (type) {
            case 0:
                for (CubeData obj : list) {
                    CubeData cubeData = cubeDataService.lambdaQuery()
                            .eq(CubeData::getAppId, targetAppId)
                            .eq(CubeData::getDatasourceId, targetAppDsid)
                            .eq(CubeData::getId, obj.getId()).one();
                    if(ObjectUtil.isEmpty(cubeData)){
                        obj.setKid(null);
                        obj.setAppId(targetAppId);
                        obj.setDatasourceId(targetAppDsid);
                        obj.setCreateTime(new Date());
                        obj.setCreateUserId(AuthUtil.getUserId());
                        cubeDataService.save(obj);
                    }else{
                        log.error("cubeDataService相关信息已经存在:{},不需要进行复制.", cubeData.toString());
                    }
                }
                break;
            case 1:
                for (CubeData obj : list) {
                    cubeDataService.removeById(obj.getKid());
                }
                break;
            default:
                throw new ValidateException("参数错误");
        }
    }


    /**
     * 给复制应用自动创建数据源(返回数据源ID)
     * @param  appCreateDatasource
     * @return Long
     */
    public Long createDatabase(AppCreateDatasourceParams appCreateDatasource) {
        String dbName = "windata_test_db_"+appCreateDatasource.getNewAppId();
        ReportDatasource newReportDatasource = dsHandler.createDatabase(dbName);
        if(ObjectUtil.isEmpty(newReportDatasource)){
             log.error("自动创建数据库失败.");
        }
        newReportDatasource.setCreateTime(new Date());
        newReportDatasource.setCreateUserId(AuthUtil.getUserId());
        newReportDatasource.setTenantId(AuthUtil.getTenantId());
        newReportDatasource.setRemark(appCreateDatasource.getNewAppName()+"应用专属数据源");
        newReportDatasource.setPoolName(IdWorker.getIdStr());
        newReportDatasource.setResponsiblePersonId(AuthUtil.getUserId());
        newReportDatasource.setResponsiblePersonName(AuthUtil.getUserName());
        newReportDatasource.setUrl(DsTypeEnum.mysql.getDriverUrlFunction().apply(newReportDatasource));
        boolean result = reportDatasourceService.save(newReportDatasource);
        log.info("newReportDatasource:{}", newReportDatasource.toString());
        if(result){
            if (!dynamicDataSourceSupport.addDs(newReportDatasource)) {
                log.error("数据源连接失败....");
            }
           return newReportDatasource.getId();
        }
        return null;
    }


    /**
     * 删除指定数据源链接信息
     * @param id
     * @return Boolean
     */
    public Boolean delReportDatabase(Long id) {
        ReportDatasource datasourceEntity = reportDatasourceService.getByDsId(id);
        if(ObjectUtil.isEmpty(datasourceEntity)){
            log.error("获取数据源连接信息失败.");
            return false;
        }
        try {
            dynamicDataSourceSupport.removeDs(datasourceEntity.getPoolName());
            log.info("Delete:DataSource|dbname={}, operator={}", datasourceEntity.getDsDb(), SecureUtil.getUserName());
            reportDatasourceService.refreshCache(Long.valueOf(datasourceEntity.getId()));
        } catch (Exception e) {
            log.error(e.getMessage(), e);
        }
        reportDatasourceService.removeById(id);
        reportDatasourcePermissionService.remove(Wrappers.<ReportDatasourcePermission>lambdaQuery().in(ReportDatasourcePermission::getDatasourceId, id));
        return true;
    }


    /**
     * 复制【数据源物理表】
     * @param  dsIds      数据源ID
     * @param  targetAppDsid 目标应用数据源ID
     * @param  platomixAppCopyLogModel  复制日志
     * @return boolean
     */
    private boolean copyAssociationTables(List<Long> dsIds, Long targetAppDsid, List<String> associationTables, PlatomixAppCopyLogModel platomixAppCopyLogModel) {
        if(CollectionUtil.isEmpty(dsIds)){
            return false;
        }
        if(CollectionUtil.isEmpty(associationTables)){
            return  false;
        }
        ReportDatasource reportDatasource = reportDatasourceService.getByDsId(targetAppDsid);
        List<TableViewDto> tableViewList = new ArrayList<>();
        for(String tableName : associationTables){
            TableViewDto tableViewDto = new TableViewDto();
            tableViewDto.setTableViewName(tableName);
            tableViewDto.setTableComments("暂无");
            tableViewDto.setDatasourceId(dsIds.get(0));
            tableViewList.add(tableViewDto);
        }
        try {
            log.info("关联物理数据表 复制参数:{}", tableViewList.toString());
            dsHandler.tableCopy(reportDatasource, tableViewList);
            platomixAppCopyLogModel.setDbResult(0);
            appClient.taskInfo(platomixAppCopyLogModel);
            log.info("关联物理数据表 复制成功.");
            return true;
        } catch (Exception e) {
            platomixAppCopyLogModel.setDbResult(1);
            platomixAppCopyLogModel.setErrorInfo(ExceptionUtil.stacktraceToString(e));
            appClient.taskInfo(platomixAppCopyLogModel);
            log.error("关联物理数据表 复制失败:" + ExceptionUtil.stacktraceToString(e));
            return false;
        }
    }
}
