/**
 *
 */
package org.springblade.resouces;

import cn.hutool.json.JSONUtil;
import lombok.extern.slf4j.Slf4j;
import org.springblade.bi.client.CubeDataClient;
import org.springblade.bi.client.form.*;
import org.springblade.bi.client.form.param.RowPermissionParam;
import org.springblade.bi.client.model.CubeBrowserSqlModel;
import org.springblade.bi.client.model.CubeDataModel;
import org.springblade.core.mp.support.Query;
import org.springblade.core.secure.BladeUser;
import org.springblade.core.secure.utils.AuthUtil;
import org.springblade.core.tool.exception.RpcException;
import org.springblade.core.tool.utils.CollectionUtil;
import org.springblade.cube.dto.BrowserParam;
import org.springblade.cube.dto.BrowserRequest;
import org.springblade.cube.model.CubeData;
import org.springblade.cube.service.CubeDataBrowserService;
import org.springblade.cube.service.CubeDataService;
import org.springblade.cube.service.MixCubeDataService;
import org.springblade.system.client.model.PlatomixAppCopyLogModel;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RestController;

import cn.hutool.core.bean.BeanUtil;

import java.util.List;
import java.util.Optional;

/**
 * @author wookvn
 */
@Slf4j
@RestController
public class CubeDataResouce implements CubeDataClient {

    @Autowired
    private CubeDataService cubeDataService;

    @Autowired
    private CubeDataBrowserService cubeDataBrowserService;

    @Autowired
    private MixCubeDataService mixCubeDataService;

    @Autowired
    private AppCopyService appCopyService;

    @Override
    public CubeDataModel getById(Long id) {
        CubeData cubeData = cubeDataService.getByBizId(id);
        cubeData.setId(cubeData.getId());
        return BeanUtil.copyProperties(cubeData, CubeDataModel.class);
    }

    @Override
    public CubeBrowserSqlModel buildBrowserSql(CubeBrowserSqlForm browserSqlForm) {
        try {
            BladeUser bladeUser = Optional.ofNullable(AuthUtil.getUser()).orElse(new BladeUser());
            bladeUser.setUserId(browserSqlForm.getUserId());
			bladeUser.setTenantId(browserSqlForm.getTenantId());

			Query pageQuery = new Query();
			pageQuery.setAscs(browserSqlForm.getAscs());
			pageQuery.setDescs(browserSqlForm.getDescs());

			BrowserRequest browserRequest = new BrowserRequest();
			browserRequest.setPageQuery(pageQuery);
			browserRequest.setParamList(BeanUtil.copyToList(browserSqlForm.getParamList(), BrowserParam.class));

            RowPermissionParam rowPermissionParam = browserSqlForm.getRowPermissionParam();
            if (browserSqlForm.getParamCondition()!=null){
                log.info("buildBrowserSql params:{}", JSONUtil.toJsonStr(browserSqlForm));

                return browserSqlForm.getTableLinkJoinParam() != null
                        ? cubeDataBrowserService.buildSqlNotParamCondition(browserSqlForm.getCubeId(), browserRequest, browserSqlForm.getTableLinkJoinParam(),browserSqlForm.getSubTableQueryParams(), bladeUser,browserSqlForm.getParamCondition(),rowPermissionParam)
                        : cubeDataBrowserService.buildSqlNotParamCondition(browserSqlForm.getCubeId(), browserRequest, bladeUser,browserSqlForm.getParamCondition(),rowPermissionParam);
            }
            log.info("buildBrowserSql tableLinkJoinParam:{}", JSONUtil.toJsonStr(browserSqlForm));
			return browserSqlForm.getTableLinkJoinParam() != null 
					? cubeDataBrowserService.buildSql(browserSqlForm.getCubeId(), browserRequest, browserSqlForm.getTableLinkJoinParam(),browserSqlForm.getSubTableQueryParams(), bladeUser,rowPermissionParam)
					: cubeDataBrowserService.buildSql(browserSqlForm.getCubeId(), browserRequest, bladeUser,rowPermissionParam);
		} catch (Exception e) {
			throw new RpcException(e.getMessage(), e);
		}
    }

    @Override
    public boolean changeCubeDataStatus(ChangeCubeDataStatusParams params) {
        if (CollectionUtil.isNotEmpty(params.getCubeDataIds())) {
            for (Long cubeDataId : params.getCubeDataIds()) {
                CubeData cubeData = new CubeData();
                cubeData.setDisplayStatus(params.isDisplay() ? 1 : 0);
                cubeData.setId(cubeDataId);
                cubeDataService.updateByBizId(cubeData);
            }
        }
        return true;
    }

    @Override
    public Long createAggCubeData(AggCubeCreateForm aggCubeCreateForm) {
        try {
            return mixCubeDataService.doSaveOrUpdate(aggCubeCreateForm);
        } catch (Exception e) {
            throw new RpcException(e.getMessage(), e);
        }
    }

    @Override
    public Boolean copy( AppCopyParams appCopyParams) {
        String appId = appCopyParams.getAppId();
        List<Long> cubeIds = appCopyParams.getCubeIds();
        List<Long> dsIds = appCopyParams.getDsIds();
        String platomixAppCopyLogModelStr = appCopyParams.getPlatomixAppCopyLogModelStr();
        List<String> associationTables = appCopyParams.getAssociationTables();
        PlatomixAppCopyLogModel platomixAppCopyLogModel = JSONUtil.toBean(platomixAppCopyLogModelStr, PlatomixAppCopyLogModel.class);
        return appCopyService.copy(cubeIds, dsIds, appId, appCopyParams.getTargetAppId(), appCopyParams.getTargetDsId(), platomixAppCopyLogModel, associationTables);
    }


    @Override
    public Boolean del(AppDelParams appDelParams) {
        String appId = appDelParams.getAppId();
        List<Long> cubeIds = appDelParams.getCubeIds();
        List<Long> dsIds = appDelParams.getDsIds();
        String platomixAppCopyLogModelStr = appDelParams.getPlatomixAppCopyLogModelStr();
        PlatomixAppCopyLogModel platomixAppCopyLogModel = JSONUtil.toBean(platomixAppCopyLogModelStr, PlatomixAppCopyLogModel.class);
        return appCopyService.del(cubeIds, dsIds, appId, platomixAppCopyLogModel);
    }

    @Override
    public Long createReportDatasource(AppCreateDatasourceParams appCreateDatasource) {
        return appCopyService.createDatabase(appCreateDatasource);
    }

    @Override
    public Boolean delReportDatasource(Long datasourceId) {
        return appCopyService.delReportDatabase(datasourceId);
    }
}
