/**
 *
 */
package org.springblade.bi.client;

import org.springblade.bi.client.form.*;
import org.springblade.bi.client.model.CubeBrowserSqlModel;
import org.springblade.bi.client.model.CubeDataModel;
import org.springblade.common.openfeign.OpenFeignConstants;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;

/**
 * @author wookvn
 * <p>
 * 数据集Client
 */
@FeignClient(contextId = "cubeDataClient", value = OpenFeignConstants.BI_SERVICE)
public interface CubeDataClient {

    /**
     * @param  id
     * @return CubeDataModel
     */
    @GetMapping(OpenFeignConstants.BI_SERVICE + "/cube/info/{id}")
    public CubeDataModel getById(@PathVariable Long id);

    /**
     * 构建低代码的数据查询SQL
     * @param  browserSqlForm
     * @return CubeBrowserSqlModel
     */
    @PostMapping(OpenFeignConstants.BI_SERVICE + "/cube/lowcode/browserSql")
    public CubeBrowserSqlModel buildBrowserSql(@RequestBody CubeBrowserSqlForm browserSqlForm);

    /**
     * 修改数据集状态
     * @param  params
     * @return boolean
     */
    @PostMapping(OpenFeignConstants.BI_SERVICE + "/cube/changeCubeDataStatus")
    public boolean changeCubeDataStatus(@RequestBody ChangeCubeDataStatusParams params);

    /**
     * 创建聚合数据集
     * @param  aggCubeCreateForm
     * @return Long
     */
    @PostMapping(OpenFeignConstants.BI_SERVICE + "/aggcube/create")
    public Long createAggCubeData(@RequestBody AggCubeCreateForm aggCubeCreateForm);

    /**
     * 数据集复制
     * @param appCopyParams
     * @return Boolean
     */
    @PostMapping(OpenFeignConstants.BI_SERVICE + "/cube/copy")
    public Boolean copy(@RequestBody AppCopyParams appCopyParams);

    /**
     * 数据集删除
     * @param appDelParams
     * @return Boolean
     */
    @PostMapping(OpenFeignConstants.BI_SERVICE + "/cube/del")
    public Boolean del(@RequestBody AppDelParams appDelParams);

    /**
     * 创建数据源(应用复制)
     * @param appCreateDatasource
     * @return Long
     */
    @PostMapping(OpenFeignConstants.BI_SERVICE + "/datasource/create")
    public Long createReportDatasource(@RequestBody AppCreateDatasourceParams appCreateDatasource);


    /**
     * 删除数据源(应用删除)
     * @param datasourceId
     * @return Long
     */
    @PostMapping(OpenFeignConstants.BI_SERVICE + "/datasource/del/{datasourceId}")
    public Boolean delReportDatasource(@PathVariable Long datasourceId);
}
